//
//	MODULE:		entry.c
//
//	PURPOSE:	Define a minimal run-time for MSVC6.0+
//
//	HISTORY:	v1.0 09/04/2002 J Brown		- Initial Version
//

#include <windows.h>

// Remove run-time library
#pragma comment (linker, "/NODEFAULTLIB:LIBC.LIB")

#ifndef _DEBUG

#pragma optimize("gsy", on)

#pragma comment(linker,"/merge:.rdata=.data")
#pragma comment(linker,"/merge:.text=.data")
#pragma comment(linker,"/merge:.reloc=.data")
#pragma comment(linker,"/FILEALIGN:0x200")

//
//	Define wrappers around c-heap-management
//
#ifdef __cplusplus

void * __cdecl operator new(unsigned int s)
{
    return HeapAlloc( GetProcessHeap(), 0, s );
}

void __cdecl operator delete( void * p )
{
    HeapFree( GetProcessHeap(), 0, p );
}

extern "C" void * __cdecl	malloc(size_t size);
extern "C" void   __cdecl	free(void * p);
extern "C" int    __cdecl	WinMainCRTStartup();

#endif	// __cplusplus

//
void * __cdecl malloc(size_t size)
{
    return HeapAlloc( GetProcessHeap(), 0, size );
}

void __cdecl free(void * p)
{
    HeapFree( GetProcessHeap(), 0, p );
}


int  __cdecl WinMainCRTStartup()
{
	UINT  ret;
	char *ptr = 0;

	HINSTANCE hInst = GetModuleHandle(0);

#ifndef NO_PARSE_CMD_LINE
	
	ptr = GetCommandLineA();

	// Skip any leading whitespace
	while(*ptr == ' ') 
		ptr++;

	// Is executable path enclosed in quotes?
	if(*ptr == '\"')
	{
		ptr++;

		while(*ptr && *ptr != '\"')
			ptr++;

		ptr++;
	}
	// No? Find next whitespace - that is where args start
	else
	{
		while(*ptr && *ptr != ' ')
			ptr++;
	}

	// find start of any arguments
	while(*ptr && *ptr == ' ')
		ptr++;

#endif

	ret = WinMain(hInst, 0, ptr, SW_SHOWNORMAL);
	ExitProcess(ret);
	return ret;
}

#endif	// _DEBUG
